@@ -1,4 +1,5 @@
-/* Copyright (c) Mark J. Kilgard, 1994. */
+
+/* Copyright (c) Mark J. Kilgard, 1994, 1997. */
 
 /* This program is freely distributable without licensing fees
    and is provided without guarantee or warrantee expressed or
@@ -9,6 +10,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#if !defined(WIN32)
 #include <X11/Xlib.h>
 #if defined(__vms)
 #include <X11/XInput.h>
@@ -16,6 +18,7 @@
 #include <X11/extensions/XInput.h>
 #endif
 #include <X11/Xutil.h>
+#endif /* !WIN32 */
 
 #include "glutint.h"
 
@@ -28,6 +31,7 @@ XDevice *__glutTablet = NULL;
 XDevice *__glutDials = NULL;
 XDevice *__glutSpaceball = NULL;
 
+#if !defined(WIN32)
 typedef struct _Range {
   int min;
   int range;
@@ -139,10 +143,12 @@ tabletPosChange(GLUTwindow * window, int first, int count, int *data)
   if (genEvent)
     window->tabletMotion(window->tabletPos[0], window->tabletPos[1]);
 }
+#endif /* !WIN32 */
 
 int
 __glutProcessDeviceEvents(XEvent * event)
 {
+#if !defined(WIN32)
   GLUTwindow *window;
 
   /* XXX Ugly code fan out. */
@@ -239,6 +245,7 @@ __glutProcessDeviceEvents(XEvent * event)
       return 1;
     }
   }
+#endif /* !WIN32 */
   return 0;
 }
 
@@ -261,6 +268,7 @@ probeDevices(void)
 {
   static Bool been_here = False;
   static int support;
+#if !defined(WIN32)
   XExtensionVersion *version;
   XDeviceInfoPtr device_info, device;
   XAnyClassPtr any;
@@ -269,11 +277,14 @@ probeDevices(void)
   XAxisInfoPtr a;
   int num_dev, btns, dials;
   int i, j, k;
+#endif /* !WIN32 */
 
   if (been_here) {
     return support;
   }
   been_here = True;
+
+#if !defined(WIN32)
   version = XGetExtensionVersion(__glutDisplay, "XInputExtension");
   if (version == NULL || ((int) version) == NoSuchExtension) {
     support = 0;
@@ -395,6 +406,9 @@ probeDevices(void)
     }
     XFreeDeviceList(device_info);
   }
+#else /* WIN32 */
+  __glutNumMouseButtons = GetSystemMetrics(SM_CMOUSEBUTTONS);
+#endif /* !WIN32 */
   /* X Input extension might be supported, but only if there is
      a tablet, dials, or spaceball do we claim devices are
      supported. */
@@ -405,6 +419,7 @@ probeDevices(void)
 void
 __glutUpdateInputDeviceMask(GLUTwindow * window)
 {
+#if !defined(WIN32)
   /* 5 (dial and buttons) + 5 (tablet locator and buttons) + 5
      (Spaceball buttons and axis) = 15 */
   XEventClass eventList[15];
@@ -503,16 +518,18 @@ __glutUpdateInputDeviceMask(GLUTwindow * window)
     /* X Input extension not supported; no chance for exotic
        input devices. */
   }
+#endif /* !WIN32 */
 }
 
 /* CENTRY */
+int APIENTRY
 glutDeviceGet(GLenum param)
 {
   probeDevices();
   switch (param) {
   case GLUT_HAS_KEYBOARD:
   case GLUT_HAS_MOUSE:
-    /* Assume X always has mouse and keyboard. */
+    /* Assume window system always has mouse and keyboard. */
     return 1;
   case GLUT_HAS_SPACEBALL:
     return __glutSpaceball != NULL;
