@@ -1,4 +1,5 @@
-/* Copyright (c) Mark J. Kilgard, 1994.  */
+
+/* Copyright (c) Mark J. Kilgard, 1994, 1997.  */
 
 /* This program is freely distributable without licensing fees
    and is provided without guarantee or warrantee expressed or
@@ -8,21 +9,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
+#if !defined(WIN32)
 #include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xatom.h>  /* for XA_RGB_DEFAULT_MAP atom */
-#if defined(__vms)
-#include <X11/StdCmap.h>  /* for XmuLookupStandardColormap */
-#else
-#include <X11/Xmu/StdCmap.h>  /* for XmuLookupStandardColormap */
-#endif
-
-/* SGI optimization introduced in IRIX 6.3 to avoid X server
-   round trips for interning common X atoms. */
-#if defined(_SGI_EXTRA_PREDEFINES) && !defined(NO_FAST_ATOMS)
-#include <X11/SGIFastAtom.h>
-#else
-#define XSGIFastInternAtom(dpy,string,fast_name,how) XInternAtom(dpy,string,how)
+#include <X11/Xatom.h>
 #endif
 
 #include <GL/glut.h>
@@ -34,6 +23,16 @@ int __glutWindowListSize = 0;
 GLUTstale *__glutStaleWindowList = NULL;
 
 void (*__glutFreeOverlayFunc) (GLUToverlay *);
+XVisualInfo *(*__glutDetermineVisualFromString) (char *string, Bool * treatAsSingle,
+  Criterion * requiredCriteria, int nRequired, int requiredMask) = NULL;
+
+static Criterion requiredWindowCriteria[] =
+{
+  {LEVEL, EQ, 0},
+  {TRANSPARENT, EQ, 0}
+};
+static int numRequiredWindowCriteria = sizeof(requiredWindowCriteria) / sizeof(Criterion);
+static int requiredWindowCriteriaMask = (1 << LEVEL) | (1 << TRANSPARENT);
 
 static void
 cleanWindowWorkList(GLUTwindow * window)
@@ -116,7 +115,7 @@ __glutGetWindow(Window win)
 }
 
 /* CENTRY */
-int
+int APIENTRY
 glutGetWindow(void)
 {
   if (__glutCurrentWindow) {
@@ -139,9 +138,11 @@ __glutSetWindow(GLUTwindow * window)
      "already bound" case, GLUT avoids the temptation to do so
      too. */
   __glutCurrentWindow = window;
+
   glXMakeCurrent(__glutDisplay, __glutCurrentWindow->renderWin,
     __glutCurrentWindow->renderCtx);
 
+#if !defined(WIN32)
   /* We should be careful to force a finish between each
      iteration through the GLUT main loop if indirect OpenGL 
      contexts are in use; indirect contexts tend to have  much
@@ -150,6 +151,7 @@ __glutSetWindow(GLUTwindow * window)
      by posting GLUT_FINISH_WORK to be done. */
   if (!__glutCurrentWindow->isDirect)
     __glutPutOnWorkList(__glutCurrentWindow, GLUT_FINISH_WORK);
+#endif
 
   /* If debugging is enabled, we'll want to check this window
      for any OpenGL errors every iteration through the GLUT
@@ -160,7 +162,7 @@ __glutSetWindow(GLUTwindow * window)
 }
 
 /* CENTRY */
-void
+void APIENTRY
 glutSetWindow(int win)
 {
   GLUTwindow *window;
@@ -217,6 +219,10 @@ getVisualInfoCI(unsigned int mode)
   int list[32];
   int i, n = 0;
 
+  /* Should not be looking at display mode mask if
+     __glutDisplayString is non-NULL. */
+  assert(!__glutDisplayString);
+
   list[n++] = GLX_BUFFER_SIZE;
   list[n++] = 1;
   if (GLUT_WIND_IS_DOUBLE(mode)) {
@@ -261,9 +267,14 @@ getVisualInfoRGB(unsigned int mode)
   int list[32];
   int n = 0;
 
+  /* Should not be looking at display mode mask if
+     __glutDisplayString is non-NULL. */
+  assert(!__glutDisplayString);
+
   /* XXX Would a caching mechanism to minize the calls to
      glXChooseVisual? You'd have to reference count
-     XVisualInfo* pointers. */
+     XVisualInfo* pointers.  Would also have to properly
+     interact with glutInitDisplayString. */
 
   list[n++] = GLX_RGBA;
   list[n++] = GLX_RED_SIZE;
@@ -339,6 +350,10 @@ __glutDetermineVisual(
 {
   XVisualInfo *vis;
 
+  /* Should not be looking at display mode mask if
+     __glutDisplayString is non-NULL. */
+  assert(!__glutDisplayString);
+
   *treatAsSingle = GLUT_WIND_IS_SINGLE(displayMode);
   vis = getVisualInfo(displayMode);
   if (!vis) {
@@ -368,95 +383,6 @@ __glutDetermineVisual(
   return vis;
 }
 
-void
-__glutSetupColormap(XVisualInfo * vi, GLUTcolormap ** colormap, Colormap * cmap)
-{
-  Status status;
-  XStandardColormap *standardCmaps;
-  int i, numCmaps;
-
-  switch (vi->class) {
-  case PseudoColor:
-    if (GLUT_WIND_IS_RGB(__glutDisplayMode)) {
-      /* Mesa might return a PseudoColor visual for RGB mode. */
-      *colormap = NULL;
-      if (MaxCmapsOfScreen(DefaultScreenOfDisplay(__glutDisplay)) == 1
-        && vi->visual == DefaultVisual(__glutDisplay, __glutScreen)) {
-        char *private = getenv("MESA_PRIVATE_CMAP");
-
-        if (private) {
-          /* User doesn't want to share colormaps. */
-          *cmap = XCreateColormap(__glutDisplay, __glutRoot,
-            vi->visual, AllocNone);
-        } else {
-          /* Share the root colormap. */
-          *cmap = DefaultColormap(__glutDisplay, __glutScreen);
-        }
-      } else {
-        /* Get our own PseudoColor colormap. */
-        *cmap = XCreateColormap(__glutDisplay, __glutRoot,
-          vi->visual, AllocNone);
-      }
-    } else {
-      /* CI mode, real GLX never returns a PseudoColor visual
-         for RGB mode. */
-      *colormap = __glutAssociateColormap(vi);
-      *cmap = (*colormap)->cmap;
-    }
-    break;
-  case TrueColor:
-  case DirectColor:
-    *colormap = NULL;   /* NULL if RGBA */
-#ifndef SOLARIS_2_4_BUG
-    /* Solaris 2.4 has a bug in its XmuLookupStandardColormap
-       implementation.  Please compile your Solaris 2.4 version
-       of GLUT with -DSOLARIS_2_4_BUG to work around this bug.
-       The symptom of the bug is that programs will get a
-       BadMatch error from X_CreateWindow when creating a GLUT
-       window because Solaris 2.4 creates a  corrupted
-       RGB_DEFAULT_MAP property.  Note that this workaround
-       prevents Colormap sharing between applications, perhaps
-       leading unnecessary colormap installations or colormap
-       flashing. */
-    status = XmuLookupStandardColormap(__glutDisplay,
-      vi->screen, vi->visualid, vi->depth, XA_RGB_DEFAULT_MAP,
-      /* replace */ False, /* retain */ True);
-    if (status == 1) {
-      status = XGetRGBColormaps(__glutDisplay, __glutRoot,
-        &standardCmaps, &numCmaps, XA_RGB_DEFAULT_MAP);
-      if (status == 1)
-        for (i = 0; i < numCmaps; i++)
-          if (standardCmaps[i].visualid == vi->visualid) {
-            *cmap = standardCmaps[i].colormap;
-            XFree(standardCmaps);
-            return;
-          }
-    }
-#endif
-    /* If no standard colormap but TrueColor, just make a
-       private one. */
-    /* XXX Should do a better job of internal sharing for
-       privately allocated TrueColor colormaps. */
-    /* XXX DirectColor probably needs ramps hand initialized! */
-    *cmap = XCreateColormap(__glutDisplay, __glutRoot,
-      vi->visual, AllocNone);
-    break;
-  case StaticColor:
-  case StaticGray:
-  case GrayScale:
-    /* Mesa supports these visuals */
-    *colormap = NULL;
-    *cmap = XCreateColormap(__glutDisplay, __glutRoot,
-      vi->visual, AllocNone);
-    break;
-  default:
-    __glutFatalError(
-      "could not allocate colormap for visual type: %d.",
-      vi->class);
-  }
-  return;
-}
-
 void
 __glutDefaultDisplay(void)
 {
@@ -489,6 +415,32 @@ __glutDefaultReshape(int width, int height)
     __glutCurrentWindow->renderCtx);
 }
 
+XVisualInfo *
+__glutDetermineWindowVisual(Bool * treatAsSingle, Bool * visAlloced)
+{
+  if (__glutDisplayString) {
+
+    /* __glutDisplayString should be NULL except if
+       glutInitDisplayString has been called to register a
+       different display string.  Calling glutInitDisplayString
+       means using a string instead of an integer mask determine
+       the visual to use. Using the function pointer variable
+       __glutDetermineVisualFromString below avoids linking in
+       the code for implementing glutInitDisplayString (ie,
+       glut_dstr.o) unless glutInitDisplayString gets called by
+       the application. */
+
+    assert(__glutDetermineVisualFromString);
+    *visAlloced = False;
+    return __glutDetermineVisualFromString(__glutDisplayString, treatAsSingle,
+      requiredWindowCriteria, numRequiredWindowCriteria, requiredWindowCriteriaMask);
+  } else {
+    *visAlloced = True;
+    return __glutDetermineVisual(__glutDisplayMode,
+      treatAsSingle, __glutGetVisualInfo);
+  }
+}
+
 GLUTwindow *
 __glutCreateWindow(GLUTwindow * parent,
   int x, int y, int width, int height)
@@ -499,29 +451,29 @@ __glutCreateWindow(GLUTwindow * parent,
   int winnum;
   int i;
 
+#if defined(WIN32)
+  WNDCLASS wc;
+  if (!GetClassInfo(GetModuleHandle(NULL), "GLUT", &wc))
+    __glutOpenWin32Connection(NULL);
+#else
   if (!__glutDisplay)
     __glutOpenXConnection(NULL);
+#endif
   winnum = getUnusedWindowSlot();
   window = (GLUTwindow *) malloc(sizeof(GLUTwindow));
   if (!window)
     __glutFatalError("out of memory.");
   window->num = winnum;
 
-  window->vis = __glutDetermineVisual(__glutDisplayMode,
-    &window->treatAsSingle, __glutGetVisualInfo);
+#if !defined(WIN32)
+  window->vis = __glutDetermineWindowVisual(&window->treatAsSingle,
+    &window->visAlloced);
   if (!window->vis) {
     __glutFatalError(
       "visual with necessary capabilities not found.");
   }
-  window->ctx = glXCreateContext(__glutDisplay, window->vis,
-    None, __glutTryDirect);
-  window->renderCtx = window->ctx;
-  window->isDirect = glXIsDirect(__glutDisplay, window->ctx);
-  if (__glutForceDirect) {
-    if (!window->isDirect)
-      __glutFatalError("direct rendering not possible.");
-  }
-  __glutSetupColormap(window->vis, &(window->colormap), &(window->cmap));
+  __glutSetupColormap(window->vis, &window->colormap, &window->cmap);
+#endif
   window->eventMask = StructureNotifyMask | ExposureMask;
 
   attribMask = CWBackPixmap | CWBorderPixel | CWColormap | CWEventMask;
@@ -537,16 +489,60 @@ __glutCreateWindow(GLUTwindow * parent,
   } else {
     wa.do_not_propagate_mask = 0;
   }
+
+  /* Stash width and height before Win32's __glutAdjustCoords
+     possibly overwrites the values. */
+  window->width = width;
+  window->height = height;
+  window->forceReshape = True;
+
+#if defined(WIN32)
+  __glutAdjustCoords(parent ? parent->win : NULL, &x, &y, &width, &height);
+  window->win = XCreateWindow(__glutDisplay,
+    parent == NULL ? __glutRoot : parent->win,
+    x, y, width, height, 0,
+    0, InputOutput, 0,
+    attribMask, &wa);
+  window->hdc = GetDC(window->win);
+  /* Must set the XHDC for fake glXChooseVisual & fake
+     glXCreateContext & fake XAllocColorCells. */
+  XHDC = window->hdc;
+  window->vis = __glutDetermineWindowVisual(&window->treatAsSingle,
+    &window->visAlloced);
+  if (!window->vis) {
+    __glutFatalError(
+      "visual with necessary capabilities not found.");
+  }
+  if (!SetPixelFormat(window->hdc,
+      ChoosePixelFormat(window->hdc, window->vis),
+      window->vis))
+    __glutFatalError("SetPixelFormat() failed in glutCreateWindow().");
+  __glutSetupColormap(window->vis, &window->colormap, &window->cmap);
+  /* Make sure subwindows get a windowStatus callback. */
+  if (parent)
+    PostMessage(parent->win, WM_ACTIVATE, 0, 0);
+#else
   window->win = XCreateWindow(__glutDisplay,
     parent == NULL ? __glutRoot : parent->win,
     x, y, width, height, 0,
     window->vis->depth, InputOutput, window->vis->visual,
     attribMask, &wa);
+#endif
   window->renderWin = window->win;
-
-  window->width = width;
-  window->height = height;
-  window->forceReshape = True;
+  window->ctx = glXCreateContext(__glutDisplay, window->vis,
+    None, __glutTryDirect);
+  if (!window->ctx) {
+    __glutFatalError(
+      "failed to create OpenGL rendering context.");
+  }
+  window->renderCtx = window->ctx;
+#if !defined(WIN32)
+  window->isDirect = glXIsDirect(__glutDisplay, window->ctx);
+  if (__glutForceDirect) {
+    if (!window->isDirect)
+      __glutFatalError("direct rendering not possible.");
+  }
+#endif
 
   window->parent = parent;
   if (parent) {
@@ -601,118 +597,24 @@ __glutCreateWindow(GLUTwindow * parent,
        buffered (in which case this is redundant, but harmless,
        because this is the initial single-buffered context
        state); or we are treating a double buffered window as a
-       single-buffered window because the system does not
-       appear to export any suitable single- buffered visuals
-       (in which the following are necessary). */
+       single-buffered window because the system does not appear
+       to export any suitable single- buffered visuals (in which
+       the following are necessary). */
     glDrawBuffer(GL_FRONT);
     glReadBuffer(GL_FRONT);
   }
   return window;
 }
 
-static int
-findColormaps(GLUTwindow * window,
-  Window * winlist, Colormap * cmaplist, int num, int max)
-{
-  GLUTwindow *child;
-  int i;
-
-  /* Do not allow more entries that maximum number of
-     colormaps! */
-  if (num >= max)
-    return num;
-  /* Is cmap for this window already on the list? */
-  for (i = 0; i < num; i++) {
-    if (cmaplist[i] == window->cmap)
-      goto normalColormapAlreadyListed;
-  }
-  /* Not found on the list; add colormap and window. */
-  winlist[num] = window->win;
-  cmaplist[num] = window->cmap;
-  num++;
-
-normalColormapAlreadyListed:
-
-  /* Repeat above but for the overlay colormap if there one. */
-  if (window->overlay) {
-    if (num >= max)
-      return num;
-    for (i = 0; i < num; i++) {
-      if (cmaplist[i] == window->overlay->cmap)
-        goto overlayColormapAlreadyListed;
-    }
-    winlist[num] = window->overlay->win;
-    cmaplist[num] = window->overlay->cmap;
-    num++;
-  }
-overlayColormapAlreadyListed:
-
-  /* Recursively search children. */
-  child = window->children;
-  while (child) {
-    num = findColormaps(child, winlist, cmaplist, num, max);
-    child = child->siblings;
-  }
-  return num;
-}
-
-void
-__glutEstablishColormapsProperty(GLUTwindow * window)
-{
-  static Atom wmColormapWindows = None;
-  Window *winlist;
-  Colormap *cmaplist;
-  Status status;
-  int maxcmaps, num;
-
-  assert(!window->parent);
-  maxcmaps = MaxCmapsOfScreen(ScreenOfDisplay(__glutDisplay,
-      __glutScreen));
-  /* For portability reasons we don't use alloca for winlist
-     and cmaplist, but we could. */
-  winlist = (Window *) malloc(maxcmaps * sizeof(Window));
-  cmaplist = (Colormap *) malloc(maxcmaps * sizeof(Colormap));
-  num = findColormaps(window, winlist, cmaplist, 0, maxcmaps);
-  if (num < 2) {
-    /* Property no longer needed; remove it. */
-    wmColormapWindows = XSGIFastInternAtom(__glutDisplay,
-      "WM_COLORMAP_WINDOWS", SGI_XA_WM_COLORMAP_WINDOWS, False);
-    if (wmColormapWindows == None) {
-      __glutWarning("Could not intern X atom for WM_COLORMAP_WINDOWS.");
-      return;
-    }
-    XDeleteProperty(__glutDisplay, window->win, wmColormapWindows);
-  } else {
-    status = XSetWMColormapWindows(__glutDisplay, window->win,
-      winlist, num);
-    /* XSetWMColormapWindows should always work unless the
-       WM_COLORMAP_WINDOWS property cannot be intern'ed.  We
-       check to be safe. */
-    if (status == False)
-      __glutFatalError("XSetWMColormapWindows returned False.");
-  }
-  /* For portability reasons we don't use alloca for winlist
-     and cmaplist, but we could. */
-  free(winlist);
-  free(cmaplist);
-}
-
-GLUTwindow *
-__glutToplevelOf(GLUTwindow * window)
-{
-  while (window->parent) {
-    window = window->parent;
-  }
-  return window;
-}
-
 /* CENTRY */
-int
-glutCreateWindow(char *title)
+int APIENTRY
+glutCreateWindow(const char *title)
 {
   static int firstWindow = 1;
   GLUTwindow *window;
+#if !defined(WIN32)
   XWMHints *wmHints;
+#endif
   Window win;
   XTextProperty textprop;
 
@@ -725,6 +627,11 @@ glutCreateWindow(char *title)
   textprop.encoding = XA_STRING;
   textprop.format = 8;
   textprop.nitems = strlen(title);
+#if defined(WIN32)
+  SetWindowText(win, title);
+  if (__glutIconic)
+    ShowWindow(win, SW_MINIMIZE);
+#else
   wmHints = XAllocWMHints();
   wmHints->initial_state =
     __glutIconic ? IconicState : NormalState;
@@ -734,29 +641,36 @@ glutCreateWindow(char *title)
     firstWindow ? __glutArgv : NULL,
     firstWindow ? __glutArgc : 0,
     &__glutSizeHints, wmHints, NULL);
-  firstWindow = 0;
   XFree(wmHints);
   XSetWMProtocols(__glutDisplay, win, &__glutWMDeleteWindow, 1);
+#endif
+  firstWindow = 0;
   return window->num + 1;
 }
 
-int
+int APIENTRY
 glutCreateSubWindow(int win, int x, int y, int width, int height)
 {
-  GLUTwindow *window, *toplevel;
+  GLUTwindow *window;
 
   window = __glutCreateWindow(__glutWindowList[win - 1],
     x, y, width, height);
-  toplevel = __glutToplevelOf(window);
-  if (toplevel->cmap != window->cmap) {
-    __glutPutOnWorkList(toplevel, GLUT_COLORMAP_WORK);
+#if !defined(WIN32)
+  {
+    GLUTwindow *toplevel;
+
+    toplevel = __glutToplevelOf(window);
+    if (toplevel->cmap != window->cmap) {
+      __glutPutOnWorkList(toplevel, GLUT_COLORMAP_WORK);
+    }
   }
+#endif
   return window->num + 1;
 }
 /* ENDCENTRY */
 
-static void
-destroyWindow(GLUTwindow * window,
+void
+__glutDestroyWindow(GLUTwindow * window,
   GLUTwindow * initialWindow)
 {
   GLUTwindow **prev, *cur, *parent, *siblings;
@@ -765,7 +679,7 @@ destroyWindow(GLUTwindow * window,
   cur = window->children;
   while (cur) {
     siblings = cur->siblings;
-    destroyWindow(cur, initialWindow);
+    __glutDestroyWindow(cur, initialWindow);
     cur = siblings;
   }
   /* Remove from parent's children list (only necessary for
@@ -809,12 +723,15 @@ destroyWindow(GLUTwindow * window,
   if (__glutWindowCache == window)
     __glutWindowCache = NULL;
 
-  XFree(window->vis);
+  if (window->visAlloced) {
+    /* Only free XVisualInfo* gotten from glXChooseVisual. */
+    XFree(window->vis);
+  }
   free(window);
 }
 
 /* CENTRY */
-void
+void APIENTRY
 glutDestroyWindow(int win)
 {
   GLUTwindow *window = __glutWindowList[win - 1];
@@ -822,7 +739,8 @@ glutDestroyWindow(int win)
   if (__glutMappedMenu && __glutMenuWindow == window) {
     __glutFatalUsage("destroying menu window not allowed while menus in use");
   }
-  /* if not a toplevel window... */
+#if !defined(WIN32)
+  /* If not a toplevel window... */
   if (window->parent) {
     /* Destroying subwindows may change colormap requirements;
        recalculate toplevel window's WM_COLORMAP_WINDOWS
@@ -830,44 +748,8 @@ glutDestroyWindow(int win)
     __glutPutOnWorkList(__glutToplevelOf(window->parent),
       GLUT_COLORMAP_WORK);
   }
-  destroyWindow(window, window);
-}
-
-void
-glutSwapBuffers(void)
-{
-  GLUTwindow *window = __glutCurrentWindow;
-
-  if (window->renderWin == window->win) {
-    if (__glutCurrentWindow->treatAsSingle) {
-      /* Pretend the double buffered window is single buffered,
-         so treat glutSwapBuffers as a no-op. */
-      return;
-    }
-  } else {
-    if (__glutCurrentWindow->overlay->treatAsSingle) {
-      /* Pretend the double buffered overlay is single
-         buffered, so treat glutSwapBuffers as a no-op. */
-      return;
-    }
-  }
-
-  /* For the MESA_SWAP_HACK. */
-  window->usedSwapBuffers = 1;
-
-  glXSwapBuffers(__glutDisplay, __glutCurrentWindow->renderWin);
-
-  /* I considered putting the window being swapped on the
-     GLUT_FINISH_WORK work list because you could call
-     glutSwapBuffers from an idle callback which doesn't call
-     __glutSetWindow which normally adds indirect rendering
-     windows to the GLUT_FINISH_WORK work list.  Not being put
-     on the list could lead to the buffering up of multiple
-     redisplays and buffer swaps and hamper interactivity.  I
-     consider this an application bug due to not using
-     glutPostRedisplay to trigger redraws.  If
-     glutPostRedisplay were used, __glutSetWindow would be
-     called and a glFinish to throttle buffering would occur. */
+#endif
+  __glutDestroyWindow(window, window);
 }
 /* ENDCENTRY */
 
@@ -892,7 +774,7 @@ __glutChangeWindowEventMask(long eventMask, Bool add)
   }
 }
 
-void
+void APIENTRY
 glutDisplayFunc(GLUTdisplayCB displayFunc)
 {
   /* XXX Remove the warning after GLUT 3.0. */
@@ -901,7 +783,7 @@ glutDisplayFunc(GLUTdisplayCB displayFunc)
   __glutCurrentWindow->display = displayFunc;
 }
 
-void
+void APIENTRY
 glutKeyboardFunc(GLUTkeyboardCB keyboardFunc)
 {
   __glutChangeWindowEventMask(KeyPressMask,
@@ -909,7 +791,7 @@ glutKeyboardFunc(GLUTkeyboardCB keyboardFunc)
   __glutCurrentWindow->keyboard = keyboardFunc;
 }
 
-void
+void APIENTRY
 glutSpecialFunc(GLUTspecialCB specialFunc)
 {
   __glutChangeWindowEventMask(KeyPressMask,
@@ -917,7 +799,7 @@ glutSpecialFunc(GLUTspecialCB specialFunc)
   __glutCurrentWindow->special = specialFunc;
 }
 
-void
+void APIENTRY
 glutMouseFunc(GLUTmouseCB mouseFunc)
 {
   if (__glutCurrentWindow->mouse) {
@@ -939,7 +821,7 @@ glutMouseFunc(GLUTmouseCB mouseFunc)
   __glutCurrentWindow->mouse = mouseFunc;
 }
 
-void
+void APIENTRY
 glutMotionFunc(GLUTmotionCB motionFunc)
 {
   /* Hack.  Some window managers (4Dwm by default) will mask
@@ -970,7 +852,7 @@ glutMotionFunc(GLUTmotionCB motionFunc)
   __glutCurrentWindow->motion = motionFunc;
 }
 
-void
+void APIENTRY
 glutPassiveMotionFunc(GLUTpassiveCB passiveMotionFunc)
 {
   __glutChangeWindowEventMask(PointerMotionMask,
@@ -985,7 +867,7 @@ glutPassiveMotionFunc(GLUTpassiveCB passiveMotionFunc)
   __glutCurrentWindow->passive = passiveMotionFunc;
 }
 
-void
+void APIENTRY
 glutEntryFunc(GLUTentryCB entryFunc)
 {
   __glutChangeWindowEventMask(EnterWindowMask | LeaveWindowMask,
@@ -996,7 +878,7 @@ glutEntryFunc(GLUTentryCB entryFunc)
   }
 }
 
-void
+void APIENTRY
 glutWindowStatusFunc(GLUTwindowStatusCB windowStatusFunc)
 {
   __glutChangeWindowEventMask(VisibilityChangeMask,
@@ -1017,7 +899,7 @@ visibilityHelper(int status)
     __glutCurrentWindow->visibility(GLUT_VISIBLE);
 }
 
-void
+void APIENTRY
 glutVisibilityFunc(GLUTvisibilityCB visibilityFunc)
 {
   __glutCurrentWindow->visibility = visibilityFunc;
@@ -1027,7 +909,7 @@ glutVisibilityFunc(GLUTvisibilityCB visibilityFunc)
     glutWindowStatusFunc(NULL);
 }
 
-void
+void APIENTRY
 glutReshapeFunc(GLUTreshapeCB reshapeFunc)
 {
   if (reshapeFunc) {
