@@ -22,7 +22,7 @@
 
 /* Some <math.h> files do not define M_PI... */
 #ifndef M_PI
-#define M_PI 3.14159265
+#define M_PI 3.14159265358979323846
 #endif
 
 int winWidth, winHeight;
@@ -149,7 +149,7 @@ loadImageTexture(void)
   GLfloat borderColor[4] =
   {1.0, 1.0, 1.0, 1.0};
 
-  if (!texData) {
+  if (!texData && texFilename) {
     imgLoad(texFilename, 2, borderColor, &texWidth, &texHeight, &texData);
   }
   if (linearFilter) {
@@ -400,8 +400,8 @@ drawSphere(void)
 
 float xmin = -0.035, xmax = 0.035;
 float ymin = -0.035, ymax = 0.035;
-float near = 0.1;
-float far = 1.9;
+float nnear = 0.1;
+float ffar = 1.9;
 float distance = -1.0;
 
 static void
@@ -417,7 +417,7 @@ loadTextureProjection(GLfloat m[16])
   glLoadIdentity();
   glTranslatef(0.5, 0.5, 0.0);
   glScalef(0.5, 0.5, 1.0);
-  glFrustum(xmin, xmax, ymin, ymax, near, far);
+  glFrustum(xmin, xmax, ymin, ymax, nnear, ffar);
   glTranslatef(0.0, 0.0, distance);
   glMultMatrixf((GLfloat *) mInverse);
   glMatrixMode(GL_MODELVIEW);
@@ -426,41 +426,41 @@ loadTextureProjection(GLfloat m[16])
 static void
 drawTextureProjection(void)
 {
-  float t = far / near;
+  float t = ffar / nnear;
   GLfloat n[4][3];
   GLfloat f[4][3];
 
   n[0][0] = xmin;
   n[0][1] = ymin;
-  n[0][2] = -(near + distance);
+  n[0][2] = -(nnear + distance);
 
   n[1][0] = xmax;
   n[1][1] = ymin;
-  n[1][2] = -(near + distance);
+  n[1][2] = -(nnear + distance);
 
   n[2][0] = xmax;
   n[2][1] = ymax;
-  n[2][2] = -(near + distance);
+  n[2][2] = -(nnear + distance);
 
   n[3][0] = xmin;
   n[3][1] = ymax;
-  n[3][2] = -(near + distance);
+  n[3][2] = -(nnear + distance);
 
   f[0][0] = xmin * t;
   f[0][1] = ymin * t;
-  f[0][2] = -(far + distance);
+  f[0][2] = -(ffar + distance);
 
   f[1][0] = xmax * t;
   f[1][1] = ymin * t;
-  f[1][2] = -(far + distance);
+  f[1][2] = -(ffar + distance);
 
   f[2][0] = xmax * t;
   f[2][1] = ymax * t;
-  f[2][2] = -(far + distance);
+  f[2][2] = -(ffar + distance);
 
   f[3][0] = xmin * t;
   f[3][1] = ymax * t;
-  f[3][2] = -(far + distance);
+  f[3][2] = -(ffar + distance);
 
   glColor3f(1.0, 1.0, 0.0);
   glBegin(GL_LINE_LOOP);
@@ -738,7 +738,8 @@ trackMotion(int x, int y)
 void
 object(void)
 {
-  static object;
+  static int object;
+
   object++;
   object %= 3;
   switch (object) {
@@ -756,16 +757,25 @@ object(void)
   }
 }
 
+static void
+nop(void)
+{
+}
+
 void
 texture(void)
 {
-  static texture;
+  static int texture = 0;
 
   texture++;
   texture %= 3;
+  if (texture == 1 && texFilename == NULL) {
+    /* Skip file texture if not loaded. */
+    texture++;
+  }
   switch (texture) {
   case 0:
-    loadTexture = NULL;
+    loadTexture = nop;
     textureEnabled = GL_FALSE;
     break;
   case 1:
@@ -862,18 +872,28 @@ usage(char *name)
   fprintf(stderr, "\n");
 }
 
+void
+menu(int selection)
+{
+  if (selection == 666) {
+    exit(0);
+  }
+  key(selection, 0, 0);
+}
+
 int
 main(int argc, char **argv)
 {
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
   (void) glutCreateWindow("projtex");
-  if (argc > 1) {
-    texFilename = argv[argc - 1];
-  } else {
+  if (argc > 2) {
     usage(argv[0]);
     exit(1);
   }
+  if (argc > 1) {
+    texFilename = argv[1];
+  }
   loadTexture = loadImageTexture;
   drawObject = drawCube;
   initialize();
@@ -883,6 +903,14 @@ main(int argc, char **argv)
   glutMouseFunc(mouse);
   glutMotionFunc(trackMotion);
   glutVisibilityFunc(vis);
+  glutCreateMenu(menu);
+  glutAddMenuEntry("Toggle showing projection", 's');
+  glutAddMenuEntry("Switch texture", 't');
+  glutAddMenuEntry("Switch object", 'o');
+  glutAddMenuEntry("Toggle filtering", 'l');
+  glutAddMenuEntry("Quit", 666);
+  glutAttachMenu(GLUT_RIGHT_BUTTON);
+  texture();
   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
 }
