@@ -45,7 +45,16 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifndef WIN32
 #include <unistd.h>
+#else
+#define R_OK 04  /* Win32 doesn't define this */
+/* Win32 defines these, but with a leading _ */
+#define pclose _pclose
+#define popen _popen
+/* Win32 doesn't have a re-entrand rand() */
+#define rand_r rand
+#endif
 #include <string.h>
 #include <math.h>
 
@@ -68,7 +77,7 @@
 /* global variables */
 
 int width, height;      /* window width, height */
-float left, right, bottom, top, near, far;  /* ortho, view volume */
+float left, right, bottom, top, nnear, ffar;  /* ortho, view volume */
 float vol_width, vol_height, vol_depth;  /* volume dimensions */
 float bminx, bmaxx, bminy, bmaxy, bminz, bmaxz, bdiag;  /* bounding box */
 int n_slices;           /* number of slices */
@@ -128,9 +137,11 @@ readVoxelData(void)
   n_slices = 128;
 
   if (tex3dSupported) {
-    vptr = (unsigned char *) malloc((vol_width * vol_height * vol_depth));
+    unsigned long size = (unsigned long) (vol_width * vol_height * vol_depth);
 
-    fread(vptr, sizeof(char), vol_width * vol_height * vol_depth, file);
+    vptr = (unsigned char *) malloc(size);
+
+    fread(vptr, sizeof(char), size, file);
     if (using_pipe) {
       pclose(file);
     } else {
@@ -139,7 +150,7 @@ readVoxelData(void)
 
     /* size of voxels is twice as the size of vptr, to duplicate alpha value
        = intensity */
-    voxels = (unsigned char *) malloc((2 * vol_width * vol_height * vol_depth));
+    voxels = (unsigned char *) malloc(2 * size);
 
     /* for now duplicate, alpha value = intensity */
     vx = voxels;
@@ -170,8 +181,8 @@ readVoxelData(void)
   right = 1.1 * bdiag;
   bottom = -1.1 * bdiag;
   top = 1.1 * bdiag;
-  near = -1.0 * bdiag;
-  far = 2 * 1.1 * bdiag;
+  nnear = -1.0 * bdiag;
+  ffar = 2 * 1.1 * bdiag;
 
   /* define the polygon dimensions on which the texture will be mapped */
   /* xy plane */
@@ -326,7 +337,7 @@ setMatrix(void)
   /* feel like using ortho projection */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  glOrtho(left, right, bottom, top, near, far);
+  glOrtho(left, right, bottom, top, nnear, ffar);
 
   /* boring view matrix */
   glMatrixMode(GL_MODELVIEW);
