@@ -1,3 +1,6 @@
+
+/* glpuzzle - written by Kevin Smith (kpsmith@engr.sgi.com) */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -96,7 +99,7 @@ static int xadds[4] =
 static int yadds[4] =
 {0, -1, 0, 1};
 
-static long W = 400, H = 300;
+static int W = 400, H = 300;
 static GLint viewport[4];
 
 #define srandom srand
@@ -763,7 +766,7 @@ selectPiece(int mousex, int mousey)
     return 0;
   }
   closest = 0;
-  dist = 4294967295;
+  dist = 4294967295U;
   while (hits) {
     if (selectBuf[(hits - 1) * 4 + 1] < dist) {
       dist = selectBuf[(hits - 1) * 4 + 1];
@@ -1125,10 +1128,10 @@ void reset(void)
       freeSolutions();
       solving = 0;
       glutChangeToMenuEntry(1, "Solving", 1);
-      glutSetWindowTitle("glpuzzle");
       movingPiece = 0;
       changeState();
     }
+    glutSetWindowTitle("glpuzzle");
     memcpy(thePuzzle, startConfig, HEIGHT * WIDTH);
     glutPostRedisplay();
 }
