@@ -7,16 +7,21 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#ifndef WIN32
 #include <unistd.h>
+#else
+#define random rand
+#define srandom srand
+#endif
 #include <math.h>
 #include <GL/glut.h>
 
 /* Some <math.h> files do not define M_PI... */
 #ifndef M_PI
-#define M_PI 3.14159265
+#define M_PI 3.14159265358979323846
 #endif
 #ifndef M_PI_2
-#define M_PI_2 1.57079632
+#define M_PI_2 1.57079632679489661923
 #endif
 
 GLboolean moving = GL_FALSE;
@@ -128,7 +133,7 @@ add_plane(void)
         SET_COLOR(0.0, 1.0, 1.0);  /* cyan */
         break;
       }
-      planes[i].speed = (random() % 20) * 0.001 + 0.02;
+      planes[i].speed = ((float) (random() % 20)) * 0.001 + 0.02;
       if (random() & 0x1)
         planes[i].speed *= -1;
       planes[i].theta = ((float) (random() % 257)) * 0.1111;
