@@ -42,17 +42,20 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <sys/types.h>
-#include <sys/time.h>
 #include <GL/glut.h>
 
 /* Some <math.h> files do not define M_PI... */
 #ifndef M_PI
-#define M_PI 3.141592654
+#define M_PI 3.14159265358979323846
 #endif
 
+#ifdef WIN32
+#define drand48() (((float) rand())/((float) RAND_MAX))
+#define srand48(x) (srand((x)))
+#else
 extern double drand48(void);
 extern void srand48(long seedval);
+#endif
 
 #define XSIZE   100
 #define YSIZE   75
