@@ -64,7 +64,7 @@ length(float v[3])
   return r;
 }
 
-static long winwidth = W, winheight = H;
+static int winwidth = W, winheight = H;
 GLUnurbsObj *nobj;
 GLuint surflist, gridlist;
 
