@@ -282,7 +282,7 @@ keyboard(unsigned char c, int x, int y)
 {
   switch (c) {
   case 'c':
-    envColor = ++envColor % NUM_ENV_COLORS;
+    envColor = ++envColor % (int) NUM_ENV_COLORS;
     break;
   case 'g':
     drawBackground = !drawBackground;
